/*
 *  Copyright (C) 2008 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * SECTION:#Kiba
 * @short_description: Class managing docks.
 * @see_also: #KibaWin,#KibaDock,#KibaObject
 * @stability: Unstable
 *
 * Manages multiple dock windows.
 */

#include "../config.h"

#include <gtk/gtk.h>

#include "kiba-win.h"
#include "kiba-dock.h"
#include "kiba-separator.h"
#include "kiba-debug.h"

#define KIBA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KIBA_TYPE, KibaPrivate))

G_DEFINE_TYPE (Kiba, kiba, KIBA_TYPE)

typedef struct {
	KibaWin *win;
	
	GList *plugins;
	
	KibaWinPositionFunc func;
	gpointer position_data;
	
	char *configFile;
}KibaDockData;

typedef struct {
	GList *docks; /* list of KibaDockData's */
}KibaPrivate;

static Kiba *kiba_instance = NULL;

/*** Private Methods ***/

static KibaDockData* __get_dock_data_for_plugin(Kiba *kiba,
                                         KibaPlugin *plugin) {
	KibaPrivate *priv = KIBA_GET_PRIVATE(kiba);
	GList *l;
	
	for (l = priv->docks; l != NULL; l = l->next) {
		KibaDockData *data;
		GList *plugins;
		for (plugins = data->plugins; plugins != NULL; plugins = plugins->next) {
			if (plugins->data == plugin)
				return data;
		}
	}
	
	return NULL;
}

static void __create_dock_win (configFile) {
	GtkWidget *window, *dock;
	GSList *dock_groups, *l;
	
	/* initiall the window */
	window = kiba_win_new ();
	DEBUG_INFO("Kiba Window created\n");
	
	/* initiall dock */
	dock = kiba_dock_new ();
	gtk_container_add (GTK_CONTAINER (window), dock);
	DEBUG_INFO("Kiba Dock created\n");
	
	dock_groups = kiba_prefs_get_list (configFile, "dock_groups");
	kiba_dock_set_group_list (KIBA_DOCK(dock), dock_groups);
	
	gtk_widget_show_all (window);
	
	/* adding separator objects */
	for (l = dock_groups; l != NULL; l = l->next) {
		GtkWidget *object;
		const char *group = l->data;
		
		if (!g_str_has_prefix (group, "separator"))
			continue;
		
		object = kiba_separator_new (group);
		gtk_container_add (GTK_CONTAINER(dock), object);
		gtk_widget_show (object);
	}
	
	return window;
}

static void __default_win_position_func () {
//TODO move the positiong stuff from KibaWin to here
}

/*** Public Methods***/

/**
 * kiba_add_plugin_object:
 * @kiba: The #Kiba instance.
 * @plugin: The #KibaPlugin instance.
 * @object: The #KibaObject to add.
 *
 * Add @object to the dock associated to @plugin.
 *
 * Returns: TRUE on success, otherwise FALSE is returned.
 */
gboolean kiba_add_plugin_object (Kiba *kiba,
                                 KibaPlugin *plugin,
                                 KibaObject *object) {
	KibaDockData *data;
	GtkWidget *dockWidget;
	
	g_return_val_if_fail(IS_KIBA(kiba), FALSE);
	g_return_val_if_fail(KIBA_IS_OBJECT(object), FALSE);
	g_return_val_if_fail(KIBA_IS_PLUGIN(plugin), FALSE);
	
	data = __get_dock_data_for_plugin(kiba, plugin);
	if (!data) {
		DEBUG_ERROR("Failed to get dock for plugin '%s' (%p)\n", plugin->id, plugin);
		return FALSE;
	}
	
	dockWidget = KIBA_WIN_CHILD(data->win);
	gtk_container_add (GTK_CONTAINER(dockWidget), object);
	
	return TRUE;
}

/**
 * kiba_get_plugin_config_file:
 * @kiba: The #Kiba instance.
 * @plugin: The #KibaPlugin instance.
 *
 * Returns path to the config file associated to plugin.
 * Can be called after 'kiba_add_object' method above was called.
 *
 * Returns: Path to the config file, or NULL if no config file was found.
 */
const char* kiba_get_plugin_config_file (Kiba *kiba,
                                         KibaPlugin *plugin) {
	KibaDockData *data;
	
	g_return_val_if_fail(IS_KIBA(kiba), NULL);
	g_return_val_if_fail(KIBA_IS_PLUGIN(plugin), NULL);
	
	data = __get_dock_data_for_plugin(kiba, plugin);
	if (!data) {
		DEBUG_ERROR("Failed to get the dock for plugin '%s' (%p)\n", plugin->id, plugin);
		return NULL;
	}
	
	return data->configFile
}

gboolean kiba_set_custom_win_position_func() {

}

void kiba_unset_custom_win_position_func() {

}

/*** Class Init/Fini ***/

static void kiba_dispose (Kiba *kiba) {
	KibaPrivate *priv = KIBA_GET_PRIVATE(kiba);
	GList *l;
	
	for (l = priv->docks; l != NULL; l = priv->docks) {
		KibaDockData *data = l->data;
		
		priv->docks = g_list_remove(priv->docks, l->data);
		
/* TODO:
		kiba_prefs_write_to_config_file ("edge_position",
			"char",
			kiba_layout_data[KIBA_DOCK(dock)->d.position].name);
		
		offset = g_strdup_printf ("%i", KIBA_DOCK(dock)->d.offset);
		kiba_prefs_write_to_config_file ("dock_edge_offset", "int", offset);
		g_free (offset);
		
		gint separator_groups = 0;
		for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
			GtkBoxChild *child = l->data;
			
			if (KIBA_IS_SEPARATOR(child->widget))
				separator_groups++;
		}
		
		i = 1;
		for (l = KIBA_DOCK(dock)->groups; l != NULL; l = l->next) {
			char *group = l->data;
			
			if (!g_str_has_prefix(group, "separator"))
				continue;
			
			if (i > separator_groups) {
				l = l->prev;
				KIBA_DOCK(dock)->groups = g_list_remove (KIBA_DOCK(dock)->groups, group);
			}
			else
				sprintf (l->data, "separator%i", i);
			
			i++;
		}
		
		groups = kiba_prefs_list_to_string(KIBA_DOCK(dock)->groups);
		kiba_prefs_write_to_config_file ("dock_groups", "list", groups);
		g_free (groups);
		
		DEBUG_MISC("Destroying the kiba widget..\n");
		gtk_widget_destroy (widget);
*/
	}
	g_list_free (priv->docks);
}

static void kiba_class_init (KibaClass *class) {
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	
	object_class->dispose = kiba_dispose;
	
	g_type_class_add_private (object_class, sizeof (KibaPrivate));
}

static void kiba_init (Kiba *kiba) {
	KibaPrivate *priv = KIBA_GET_PRIVATE(kiba);
	KibaPlugin Loader *loader;
	char *configPath, *userPath;
	GDir *configDir;
	
	priv->docks = NULL;
	
	/* create kiba home directory */
	userPath = g_strdup_printf ("%s/.kiba-dock", g_get_home_dir ());
	if (!g_file_test (userPath, G_FILE_TEST_IS_DIR)) {
		if (g_mkdir (userPath, 7664) != 0) {
			DEBUG_ERROR ("Failed to create user directory at '%s'!\n", userPath);
			exit(0);
		}
	}
	
	/* create config directory */
	configPath = g_strdup_printf ("%s/config", userPath);
	g_free(userPath);
	if (!g_file_test (configPath, G_FILE_TEST_IS_DIR)) {
		if (g_file_test (configPath, G_FILE_TEST_IS_REGULAR))
			g_remove(configPath);
		if (g_mkdir (configPath, 7664) != 0) {
			DEBUG_ERROR ("Failed to create config directory at '%s'!\n", configPath);
			g_free(configPath);
			exit(0);
		}
	}
	
	/* open the config directory for reading */
	configDir = g_dir_open (configPath, 0, NULL);
	if (!configDir) {
		DEBUG_ERROR ("Failed to open config directory at '%s'!\n", configPath);
		g_free(configPath);
		exit(0);
	}
	
	/* if the config dir is empty, create the default config file */
	if (g_dir_read_name(configDir) == NULL) {
		char *defaultFile = g_strdup_printf("%s/default.xml", configPath);
		kiba_prefs_create_config_file(defaultFile);
		g_free(defaultFile);
	}
	g_free(configPath);
	g_dir_rewind (configDir);
	
	loader = kiba_plugin_loader_get();
	
	/* get all config files from @configDir */
	while () {
		KibaDockData *data;
		GtkWidget *dock;
		const char *configFile = g_dir_read_name(configDir);
		GList *l;
		
		/* quit loop if there are no more files */
		if (!configFile) break;
		
		if (!g_str_has_suffix(configFile, ".xml"))
			continue;
		
		/* add the config file to the database */
		kiba_prefs_add_config_file(file);
		/* update the file from schema */
		kiba_prefs_update_config_file(file, kiba_prefs_get());
		
		data = g_new0(KibaDockData, 1);
		data->func = __default_win_position_func;
		data->position_data = NULL;
		data->plugins = NULL;
		data->configFile = g_strdup(configFile);
		
		/* create the dock */
		data->win = __create_dock_win(file);
		dock = KIBA_WIN_CHILD(data->win);
		
		priv->docks = g_list_append(priv->docks, data);
		
		/* add callbacks for this file */
		kiba_prefs_callbacks_add_dock (file, KIBA_WIN(window), KIBA_DOCK(dock));
		
// TODO
// 		__set_default_win_position_func()
		
		//TODO plugin stuff
		
		for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
			GtkBoxChild *child = l->data;
			/* avoids scaling of objects */
			KIBA_OBJECT(child->widget)->destroy_factor = 1.0;
		}
	}
	g_dir_close (configDir);
}

static Kiba* kiba_new() {
	return g_object_new(KIBA_TYPE, NULL);
}

/**
 * kiba_get:
 *
 * Returns the #Kiba instance.
 * Don't call g_object_unref on it, this is done only once on exiting.
 *
 * Returns: A Pointer to the #Kiba instance.
 */
Kiba* = kiba_get() {
	if (!kiba_instance)
		kiba_instance = kiba_new();
	return kiba_instance;
}
